
#ifdef SWIGGO

/* -----------------------------------------------------------------------------
 *
 *
 *
 * ----------------------------------------------------------------------------- */

//
%{
#include <infospace/Odms.hpp>

static const int DocIDToInt(lemur::api::DOCID_T id) {return id;}
static const lemur::api::DOCID_T IntToDocID(int id) {return id;}
%}
static const int DocIDToInt(lemur::api::DOCID_T id) {return id;}
static const lemur::api::DOCID_T IntToDocID(int id) {return id;}

#include <infospace/Odms.hpp>

#define VALUE INT64

namespace ride
{
  /// API definitions.
  namespace api
  {
    setEx(InfoSpace::InfoSpace);
    setEx(InfoSpace::~InfoSpace);
	setEx(InfoSpace::IsAttached);
	setEx(InfoSpace::IsDetached);
	setEx(InfoSpace::InitSchema);
	setEx(InfoSpace::VerifySchemaPostInit);
	setEx(InfoSpace::ReclaimSchema);
	setEx(InfoSpace::InitCodecFactory);
	setEx(InfoSpace::LookupCodecFactory);
	setEx(InfoSpace::GenROMeta);
	setEx(InfoSpace::GenRWMeta);
	setEx(InfoSpace::GetWindow);
	setEx(InfoSpace::GetWindowEnd);
	setEx(InfoSpace::GetDes);
	setEx(InfoSpace::GetApp);
	setEx(InfoSpace::GetName);
	setEx(InfoSpace::GetServer);
	setEx(InfoSpace::GetCommand);
	setEx(InfoSpace::GetIndexPath);
	setEx(InfoSpace::GetCorpusPath);
	setEx(InfoSpace::GetCorpusClass);
	setEx(InfoSpace::GetBuildParam);
	setEx(InfoSpace::GetStopParam);
	setEx(InfoSpace::Attach);
	setEx(InfoSpace::Detach);
	setEx(InfoSpace::Destroy);
	setEx(InfoSpace::AddDoc);
	setEx(InfoSpace::AddFile);
	setEx(InfoSpace::AddString);
	setEx(InfoSpace::DeleteDomain);
	setEx(InfoSpace::EncodeTime);
	setEx(InfoSpace::DecodeTime);
	setEx(InfoSpace::GetPos);
	setEx(InfoSpace::SetPos);
	setEx(InfoSpace::GetMax);
	setEx(InfoSpace::SetMax);
	setEx(InfoSpace::CreateCodecSet);
	setEx(InfoSpace::DestroyCodecSet);
	setEx(InfoSpace::NFLess);
	setEx(InfoSpace::NFGreater);
	setEx(InfoSpace::NFBetween);
	setEx(InfoSpace::NFEquals);
	setEx(InfoSpace::Find);
	setEx(InfoSpace::DomainExists);
	setEx(InfoSpace::GetDomain);
	setEx(InfoSpace::GetDomains);
	setEx(InfoSpace::FindExtent);

	typedef struct {
	  time_t rawtime;
	  std::string baseTime;
	  // map metadata and field names found in index params file to codecs
	  std::vector< std::pair< std::string, ride::codec::CodecFactory* > > ro_meta;
	  std::vector< std::pair< std::string, ride::codec::CodecFactory* > > rw_meta;
	  std::vector< std::pair< std::string, ride::codec::CodecFactory* > > fields;
	} Schema;

	typedef struct {
	  // document metadata and field properties
	  std::vector< std::pair< std::string, ride::codec::Codec* > > ro_meta;
	  std::vector< std::pair< std::string, ride::codec::Codec* > > rw_meta;
	  std::vector< std::pair< std::string, ride::codec::Codec* > > fields;
	} FieldProps;

    /*
     * InfoSpace Manager
     */
    class InfoSpace {

    public:
      /*
       * InfoSpace management methods
       */
      InfoSpace(  std::string app="vectavi",
                  std::string space="tx",
                  std::string server="",
                  std::string command="VankBuildIndex",
                  std::string index_path="/home/tavit/Work/repo/base",
                  std::string corpus_path="/home/tavit/Work/repo/base/corpus",
                  std::string corpus_class="html",
                  std::string build_param="/home/tavit/Work/repo/config/param/tx",
                  std::string stop_param="/home/tavit/Work/repo/config/param/smallstop_param"
                );
      ~InfoSpace();

      bool isAttached();
      bool isDetached();

      void initSchema();                                // initialize schema
      void verifySchemaPostInit();                      // verify schema post inititialization
      void reclaimSchema();                             // reclaim schema storage
      ride::codec::CodecFactory* initCodecFactory( std::string fieldName, std::string enc, INT64 max, INT64 res );
      ride::codec::CodecFactory* lookupCodecFactory( std::string fieldName );
      void genROMeta( std::vector< std::pair< std::string, ride::codec::Codec* > >* ro_meta );
      void genRWMeta( std::vector< std::pair< std::string, ride::codec::Codec* > >* rw_meta );

      VALUE getWindow();                                // InfoSpace partition Window start
      VALUE getWindowEnd();                             // InfoSpace partition Window end
      ride::api::Des* getDes();                         // InfoSpace discourse encoding sublayer
      std::string getApp();                             // Application name
      std::string getName();                            // InfoSpace name
      std::string getServer();                          // InfoSpace server
      std::string getCommand();                         // InfoSpace command
      std::string getIndexPath();                       // InfoSpace index path
      std::string getCorpusPath();                      // InfoSpace corpus path
      std::string getCorpusClass();                     // InfoSpace corpus class
      std::string getBuildParam();                      // InfoSpace build param
      std::string getStopParam();                       // InfoSpace stop param

      std::string attach();                             // create or open this infoSpace
      std::string detach();                             // close this infoSpace
      std::string destroy();                            // destroy this infoSpace

      /*
       * Index methods
       */
      lemur::api::DOCID_T addDoc( std::string fileName, bool isString = false );
      lemur::api::DOCID_T addFile( std::string fileName );
      lemur::api::DOCID_T addString( std::string documentString );
      lemur::api::DOCID_T deleteDomain( lemur::api::DOCID_T id );

      std::string encodeTime( VALUE time );
      VALUE decodeTime( std::string time );

      /*
       * Query methods
       */
      VALUE getPos();                                   // get query return cursor position
      VALUE setPos( VALUE pos );                        // set query return cursor position
      VALUE getMax();                                   // get max query return count
      VALUE setMax( VALUE count );                      // set max query return count

      ride::codec::Codec* createCodecSet( std::string fieldName, std::string value );
      void destroyCodecSet( ride::codec::Codec* compositeFieldCodec );
      std::string nFLess( std::string fieldName, std::string value );
      std::string nFGreater( std::string fieldName, std::string value );
      std::string nFBetween( std::string fieldName, std::string value1, std::string value2 );
      std::string nFEquals( std::string fieldName, std::string value );

      std::vector< lemur::api::DOCID_T > find( std::string q );         // query and return internal IDs
	  bool domainExists( lemur::api::DOCID_T );                    // check existance of document in repository
      ride::api::Domain* getDomain( lemur::api::DOCID_T );
                                                        // return domain object
      std::vector<ride::api::Domain*> getDomains( std::vector< lemur::api::DOCID_T > ids );
                                                        // return an array of domain objects

      std::vector<std::string> findExtent( std::string q, std::string f, INT64 m );
                                                        // query and return field extent for matching docs

    };

	setEx(Domain::Domain);
	setEx(Domain::ReclaimProps);
	setEx(Domain::GetFd);
	setEx(Domain::SetFd);
	setEx(Domain::GetFds);
	setEx(Domain::GetMeta);
	setEx(Domain::SetMeta);
	setEx(Domain::GetROMeta);
	setEx(Domain::GetRWMeta);
	setEx(Domain::GetCursor);
	setEx(Domain::SetCursor);
	setEx(Domain::GetField);
	setEx(Domain::SetField);
	setEx(Domain::AddField);
	setEx(Domain::GetFields);
	setEx(Domain::GetFieldCount);
	setEx(Domain::PrintTagIndent);
	setEx(Domain::PrintTagOpenValue);
	setEx(Domain::PrintTagClose);
	setEx(Domain::PrintNode);
	setEx(Domain::RenderToString);
	setEx(Domain::Save);
	setEx(Domain::SaveToFile);

	/*
	 * Model Object
	 */
	class Domain {

	private:

	  InfoSpace* _space;
	  FieldProps _properties;

	  int _cursor;

	public:

	  /*
	   * Domain methods
	   */
	  Domain( InfoSpace* space );
	  ~Domain();
	  void reclaimProps();  // reclaim document property storage


	  //    Common methods for metadata and field attributes
	  bool getFd( std::vector< std::pair< std::string, ride::codec::Codec* > >& fds,
				  std::string name, std::string& s, int& cursor );
	  bool setFd( std::vector< std::pair< std::string, ride::codec::Codec* > >& fds,
				  std::string name, std::string& s, int& cursor );
	  std::vector< std::pair< std::string, std::string > > getFds( std::vector< std::pair< std::string, ride::codec::Codec* > >& fds );

	  /*
	   *    Domain metadata methods
	   */
	  std::string getMeta( std::string name, int cursor=0 );        // get metadata text
	  std::string setMeta( std::string name, std::string s, int cursor=0 ); // set metadata text
	  std::vector< std::pair< std::string, std::string > > getROMeta();
																	// get all read-only metadata entries
	  std::vector< std::pair< std::string, std::string > > getRWMeta();
																	// get all read-write metadata entries

	  /*
	   *    Domain field methods
	   */
	  int getCursor();
	  int setCursor( int c );
	  std::string getField( std::string name, int next=0 );       // get field text
	  std::string setField( std::string name, std::string s, int next=0 );  // set field text
	  std::string addField( std::string name, std::string s, int parentOrdinal=0 );
	  std::vector< std::pair< std::string, std::string > > getFields();
																	// get all read-write field entries
	  int getFieldCount();                                        // get current count of fields

	  /*
	   * Index methods
	   */
	  void printTagIndent( std::stringstream& ds, int depth );
	  void printTagOpenValue( std::stringstream& ds, std::pair< std::string, std::string > p, int depth );
	  void printTagClose( std::stringstream& ds, std::pair< std::string, std::string > p, int depth );
	  void printNode(   // print tag hierarchy given ordered lists of parents and children
		std::stringstream& ds,
		std::vector< std::set< int >* >& parents,
		std::map< int, std::pair< std::string, std::string > >& fdMap,
		int ordinal,
		int depth );
	  std::string renderToString();                                 // render document and return as string
	  lemur::api::DOCID_T save();                                                 // index document and store in repository
	  std::string saveToFile( std::string fileName );               // write document to a corpus directory file

	};

  } // namespace api

} // namespace ride


#endif
