# go-idx-engine

A go module that provides the GO interface to the _infospace_ and _vank_ libraries that implement _infospace management_ using a _search engine_.

------------------
Build instructions
------------------

Pre-Requisites

1) The vank library should be built first and installed in the prefix directory $MY_INSTALL_HOME specified below.

2) The infospace library should be built and installed in the prefix directory $MY_INSTALL_HOME specified below.

Linux platform

```bash
cd $PROJECT_ROOT

#
# swig interface differences between vank and go-idx-engine should match the
# enumerated list of expected differences in the file: INTERFACE-DIFFS.md
#
# use the following to find differences in common interface files
(cd ~/Work/vank/swig/src/interface; ls *.i) | xargs -n1 -t -iXXX diff XXX ./XXX
# next command line should print 40
ls *.i | wc -l
# next command line should print 38
ls ~/Work/vank/swif/src/interface/*.i | wc -l
# differences from next command should match those expected: INTERFACE-DIFFS.md
ls *.i | xargs -n1 -t -iXXX diff XXX ~/Work/vank/swig/src/interface/XXX
#
# correct as appropriate, depending on whether changes are applied/verified in
# vank of go-idx-engine folder.
#

# copy CGO header makefile
cp -v ~/usr/share/vank/Makefile.cgo ./
cp -v ~/usr/share/infospace/Makefile.icgo ./

make -f Makefile.cgo clean
make -f Makefile.icgo clean
make -f Makefile.cgo
make -f Makefile.icgo
go clean

# golang security precaution requires next line
# see: https://zchee.github.io/golang-wiki/InvalidFlag/
export CGO_CXXFLAGS_ALLOW="-DPACKAGE_BUGREPORT=tavit@dms3.io"

# repeat the next three commands when test command fails as discussed below
# and fixes for "could not determine kind of name" are applied as discussed
# under notes below.
swig -go -cgo -c++ -intgosize 64 vank.i
go build -a -v
go test -v

### see notes "i)" and "ii)" below if test fails with:
# gitlab.dms3.io/is/go-idx-engine
./vank_go.go:6253:58: could not determine kind of name for C._wrap_MetadataPairVector_get_vank_go_073f147d1da00b5b
./vank_go.go:6814:58: could not determine kind of name for C._wrap_Wrapped_InfoSpace_addDoc__SWIG_0_vank_go_073f147d1da00b5b
./vank_go.go:6801:58: could not determine kind of name for C._wrap_Wrapped_InfoSpace_addDoc__SWIG_1_vank_go_073f147d1da00b5b
./vank_go.go:6931:43: could not determine kind of name for C._wrap_Wrapped_InfoSpace_getDomain_vank_go_073f147d1da00b5b
./vank_go.go:7227:2: could not determine kind of name for C._wrap_delete_DocIdVector_vank_go_073f147d1da00b5b
./vank_go.go:7296:2: could not determine kind of name for C._wrap_delete_StringPairVector_vank_go_073f147d1da00b5b
./vank_go.go:7182:54: could not determine kind of name for C._wrap_new_DocIdVector__SWIG_0_vank_go_073f147d1da00b5b
./vank_go.go:7187:54: could not determine kind of name for C._wrap_new_DocIdVector__SWIG_1_vank_go_073f147d1da00b5b
./vank_go.go:6208:68: could not determine kind of name for C._wrap_new_MetadataPairVector__SWIG_0_vank_go_073f147d1da00b5b
./vank_go.go:6213:68: could not determine kind of name for C._wrap_new_MetadataPairVector__SWIG_1_vank_go_073f147d1da00b5b
./vank_go.go:7251:64: could not determine kind of name for C._wrap_new_StringPairVector__SWIG_0_vank_go_073f147d1da00b5b
./vank_go.go:7256:64: could not determine kind of name for C._wrap_new_StringPairVector__SWIG_1_vank_go_073f147d1da00b5b
./vank_go.go:6950:46: could not determine kind of name for C._wrap_new_Wrapped_Domain_vank_go_073f147d1da00b5b
./vank_go.go:5654:66: could not determine kind of name for C._wrap_new_Wrapped_IndexEnvironment_vank_go_073f147d1da00b5b
./vank_go.go:6338:49: could not determine kind of name for C._wrap_new_Wrapped_InfoSpace__SWIG_0_vank_go_073f147d1da00b5b
./vank_go.go:6383:49: could not determine kind of name for C._wrap_new_Wrapped_InfoSpace__SWIG_1_vank_go_073f147d1da00b5b
./vank_go.go:6424:49: could not determine kind of name for C._wrap_new_Wrapped_InfoSpace__SWIG_2_vank_go_073f147d1da00b5b
./vank_go.go:6461:49: could not determine kind of name for C._wrap_new_Wrapped_InfoSpace__SWIG_3_vank_go_073f147d1da00b5b
./vank_go.go:6494:49: could not determine kind of name for C._wrap_new_Wrapped_InfoSpace__SWIG_4_vank_go_073f147d1da00b5b
./vank_go.go:6523:49: could not determine kind of name for C._wrap_new_Wrapped_InfoSpace__SWIG_5_vank_go_073f147d1da00b5b
./vank_go.go:6548:49: could not determine kind of name for C._wrap_new_Wrapped_InfoSpace__SWIG_6_vank_go_073f147d1da00b5b
./vank_go.go:6569:49: could not determine kind of name for C._wrap_new_Wrapped_InfoSpace__SWIG_7_vank_go_073f147d1da00b5b
./vank_go.go:6586:49: could not determine kind of name for C._wrap_new_Wrapped_InfoSpace__SWIG_8_vank_go_073f147d1da00b5b
./vank_go.go:6599:49: could not determine kind of name for C._wrap_new_Wrapped_InfoSpace__SWIG_9_vank_go_073f147d1da00b5b
./vank_go.go:6086:61: could not determine kind of name for C._wrap_new_Wrapped_MetadataPair_vank_go_073f147d1da00b5b
./vank_go.go:5603:51: could not determine kind of name for C._wrap_new_Wrapped_Parameters_vank_go_073f147d1da00b5b
FAIL	gitlab.dms3.io/is/go-idx-engine [build failed]

```

__Notes__
- the cgo directives are generated by Makefilie.cgo from vank, and Makefile.icgo from infospace. the swig compile command adds the directives to the top of vank_go.go as __import "C"__.

- when the primary interface file vank.i is modified, swig compilation may assign go type names a different unique numeric sub-string referring to the wrapped c++ types that cause a subsequent go build to fail. When this happens, interface files that define go types for c++ types need to be modified and re-compiled with swig. the following patterns highlight the substring that should match:

	i) find the types assigned new unique number:

```bash
egrep --color '_vank_go_[0123456789abcdef]{16}' vank_go.go
extern void _wrap_Swig_free_vank_go_c2859e9b02de2bf9(uintptr_t arg1);
```

	ii) edit the references to those types in referencing interface files:

```bash
egrep --color '_vank_go_[0123456789abcdef]{16}' *.i
IndexEnvironment_post.i:    return (IndexEnvironment)(SwigcptrWrapped_IndexEnvironment(C._wrap_new_Wrapped_IndexEnvironment_vank_go_8e24520e0b567faa()))
```

	iii) re-compile using the swig command above.

- this module wraps c++ exceptions and converts them into GO errors. the mechanism relies on c++ and CGO implementations and is not guaranteed to always properly intercept the c++ side exception. the mechanism used is described in the two interface files:
	```bash
	protect_post.i
	protect_pre.i
	```

- the following vank interface files __must__ match vank/swig/src/interface/:

	```bash
	ConflationPattern.i
	DocIdVector_post.i
	DocumentVector.i
	greedy_vector.i
	IndexEnvironment.i
	IndexEnvironment_post.i
	IntVector.i
	LemurException.i
	MetadataPair.i
	MetadataPair_post.i
	MetadataPairVector.i
	MetadataPairVector_post.i
	MResults.i
	Parameters.i
	Parameters_post.i
	ParsedDocument.i
	QueryAnnotationNode.i
	QueryEnvironment.i
	QueryExpander.i
	QueryRequest.i
	QueryResults.i
	ScoredExtentArray.i
	SnippetBuilder.i
	Specification.i
	Specification_post.i
	StringMap.i
	StringPair.i
	StringPairVector.i
	StringPairVector_post.i
	StringVector.i
	VankBuildIndex.i
	VankBuildIndex_post.i
	vank.i
	vanktypemaps.i
	```
- the following vank interface files parts that reference infospace are commented out in vank/swig/src/interface/:

	```bash
	DocIdVector.i
	DomainPtrVector.i
	protect_post.i
	protect_pre.i
	```

- the following infospace interface files __must__ match infospace/swig/src/interface/:

	```bash
	Odms.i
	Odms_post.i
	```
