package vank_go

import (
	//"errors"
	"fmt"
	//"io"
	"io/ioutil"
	"os"
	"path/filepath"
	"regexp"
	"strings"
	"testing"
	"unicode"
)

func stripSpaces(str string) string {
	return strings.Map(func(r rune) rune {
		if unicode.IsSpace(r) {
			// if the character is a space, drop it
			return -1
		}
		// else keep it in the string
		return r
	}, str)
}

/**
 *
 * This test verifies the infospace interface
 *
**/
func TestInfoSpaceLifecycle(t *testing.T) {
	//var err error = nil
	err := testInfoSpaceLifecycle()
	if err != nil {
		t.Fatal(err)
	}
}

//
// start of test logic implimentations
//

/*

Refer to Odms_post.i for the full list of index environment and domain/document
interface to cover in this test module.

* not part of the Public API, do not use.
& indirectly exercised by other public API.

	InfoSpaceInitSchema * 			- indirectly called during attach()
	InfoSpaceVerifySchemaPostInit *	- indirectly called during attach()
	InfoSpaceReclaimSchema *		- indirectly called during DeleteInfoSpace()
	InfoSpaceInitCodecFactory *
	InfoSpaceLookupCodecFactory *
	InfoSpaceGenROMeta *
	InfoSpaceGenRWMeta *
	InfoSpaceGetDes *
	InfoSpaceAddDoc &	             - called by AddFile/AddString
	InfoSpaceEncodeTime *
	InfoSpaceDecodeTime *
	InfoSpaceCreateCodecSet *
	InfoSpaceDestroyCodecSet *

	DomainReclaimProps *
	DomainGetFd *
	DomainSetFd *
	DomainGetFds *
	DomainPrintTagIndent * - indirectly called during DomainRenderToString()
	DomainPrintTagOpenValue * - indirectly called during DomainRenderToString()
	DomainPrintTagClose * - indirectly called during DomainRenderToString()
	DomainPrintNode * - indirectly called during DomainRenderToString()

*/

func testInfoSpaceLifecycle() (err error) {

	defer catch(&err)

	dir, err := ioutil.TempDir("", "test-repo-root")
	if err != nil {
		err = fmt.Errorf("failed to create TempDir: %v", err)
		return
	}
	defer os.RemoveAll(dir) // clean up

	var repositoryPath = filepath.Join(dir, "index-1")
	var corpusPath = filepath.Join(dir, "corpus-1")
	var corpusFileClass = "html"

	err = os.Mkdir(repositoryPath, 0775)
	if err != nil {
		err = fmt.Errorf("failed to create repository dir: %v", err)
		return
	}

	err = os.Mkdir(corpusPath, 0775)
	if err != nil {
		err = fmt.Errorf("failed to create corpus dir: %v", err)
		return
	}

	var is InfoSpace

	var a_app string = "infospace_test.go"
	var a_space string = "blog"
	var a_server string = ""
	var a_command string = "VankBuildIndex"
	var a_index_path string = repositoryPath
	var a_corpus_path string = corpusPath
	var a_corpus_class string = corpusFileClass
	var a_build_param string = "data/params.xml"
	var a_stop_param string = "data/smallstop_param"
	is, err = NewInfoSpace(a_app,
		a_space,
		a_server,
		a_command,
		a_index_path,
		a_corpus_path,
		a_corpus_class,
		a_build_param,
		a_stop_param)
	/*
		var a_app string = "vectavi"
		var a_space string = "tx"
		var a_server string = ""
		var a_command string = "VankBuildIndex"
		var a_index_path string = "/home/tavit/Work/repo/base"
		var a_corpus_path string = "/home/tavit/Work/repo/base/corpus"
		var a_corpus_class string = "html"
		var a_build_param string = "/home/tavit/Work/repo/config/param/" + a_app + "-" + a_space
		var a_stop_param string = "/home/tavit/Work/repo/config/param/smallstop_param"
		is, err = NewInfoSpace( a_app,
								a_space,
								a_server,
								a_command,
								a_index_path,
								a_corpus_path,
								a_corpus_class,
								a_build_param,
								a_stop_param )
	*/
	if err != nil {
		return
	}
	defer DeleteInfoSpace(is)

	r_appName, err := is.InfoSpaceGetApp()
	if err != nil {
		return
	}
	if r_appName != a_app {
		err = fmt.Errorf("mismatched InfoSpaceGetApp %v\n", r_appName)
		return
	}

	r_isName, err := is.InfoSpaceGetName()
	if err != nil {
		return
	}
	if r_isName != a_space {
		err = fmt.Errorf("mismatched InfoSpaceGetName %v\n", r_isName)
		return
	}

	r_server, err := is.InfoSpaceGetServer()
	if err != nil {
		return
	}
	if r_server != a_server {
		err = fmt.Errorf("mismatched InfoSpaceGetServer %v\n", r_server)
		return
	}

	r_command, err := is.InfoSpaceGetCommand()
	if err != nil {
		return
	}
	if r_command != a_command {
		err = fmt.Errorf("mismatched InfoSpaceGetCommand %v\n", r_command)
		return
	}

	r_indexPath, err := is.InfoSpaceGetIndexPath()
	if err != nil {
		return
	}
	if r_indexPath != a_index_path {
		err = fmt.Errorf("mismatched InfoSpaceGetIndexPath %v\n", r_indexPath)
		return
	}

	r_corpusPath, err := is.InfoSpaceGetCorpusPath()
	if err != nil {
		return
	}
	if r_corpusPath != a_corpus_path {
		err = fmt.Errorf("mismatched InfoSpaceGetCorpusPath %v\n", r_corpusPath)
		return
	}

	r_corpusClass, err := is.InfoSpaceGetCorpusClass()
	if err != nil {
		return
	}
	if r_corpusClass != a_corpus_class {
		err = fmt.Errorf("mismatched InfoSpaceGetCorpusClass %v\n", r_corpusClass)
		return
	}

	r_paramsFilePath, err := is.InfoSpaceGetBuildParam()
	if err != nil {
		return
	}
	if r_paramsFilePath != a_build_param {
		err = fmt.Errorf("mismatched InfoSpaceGetBuildParam %v\n", r_paramsFilePath)
		return
	}

	r_stopWordsFilePath, err := is.InfoSpaceGetStopParam()
	if err != nil {
		return
	}
	if r_stopWordsFilePath != a_stop_param {
		err = fmt.Errorf("mismatched InfoSpaceGetStopParam %v\n", r_stopWordsFilePath)
		return
	}

	r_isAttached, err := is.InfoSpaceIsAttached()
	if err != nil {
		return
	}
	if r_isAttached {
		err = fmt.Errorf("expected InfoSpaceIsAttached to be false, but is %v\n", r_isAttached)
		return
	}

	_, err = is.InfoSpaceAttach()
	if err != nil {
		return
	}

	r_isDetached, err := is.InfoSpaceIsDetached()
	if err != nil {
		return
	}
	if r_isDetached {
		err = fmt.Errorf("expected InfoSpaceIsDetached to be false, but is %v\n", r_isDetached)
		return
	}

	r_window, err := is.InfoSpaceGetWindow()
	if err != nil {
		return
	}
	if r_window <= 0 {
		err = fmt.Errorf("expected InfoSpaceGetWindow to be non-zero, but is %v\n", r_window)
		return
	}

	r_windowEnd, err := is.InfoSpaceGetWindowEnd()
	if err != nil {
		return
	}
	if r_windowEnd <= 0 {
		err = fmt.Errorf("expected InfoSpaceGetWindowEnd to be non-zero, but is %v\n", r_windowEnd)
		return
	}

	r_detach, err := is.InfoSpaceDetach()
	if err != nil {
		return
	}
	if len(r_detach) <= 0 {
		err = fmt.Errorf("expected InfoSpaceDetach result to be non-empty, but is %v\n", r_detach)
		return
	}

	r_destroy, err := is.InfoSpaceDestroy()
	if err != nil {
		return
	}
	if len(r_destroy) <= 0 {
		err = fmt.Errorf("expected InfoSpaceDestroy result to be non-empty, but is %v\n", r_destroy)
		return
	}

	// re-attach to destroyed index (should create new index)
	fmt.Printf("testInfoSpaceLifecycle: re-attach to destroyed index (should create new index)\n")

	// we first need to create index path again
	err = os.Mkdir(repositoryPath, 0775)
	if err != nil {
		err = fmt.Errorf("failed to create repository dir: %v", err)
	}

	_, err = is.InfoSpaceAttach()
	if err != nil {
		return
	}

	r_isDetached, err = is.InfoSpaceIsDetached()
	if err != nil {
		return
	}
	if r_isDetached {
		err = fmt.Errorf("expected InfoSpaceIsDetached to be false, but is %v\n", r_isDetached)
		return
	}

	r_window, err = is.InfoSpaceGetWindow()
	if err != nil {
		return
	}
	if r_window <= 0 {
		err = fmt.Errorf("expected InfoSpaceGetWindow to be non-zero, but is %v\n", r_window)
		return
	}

	r_windowEnd, err = is.InfoSpaceGetWindowEnd()
	if err != nil {
		return
	}
	if r_windowEnd <= 0 {
		err = fmt.Errorf("expected InfoSpaceGetWindowEnd to be non-zero, but is %v\n", r_windowEnd)
		return
	}

	r_detach, err = is.InfoSpaceDetach()
	if err != nil {
		return
	}
	if len(r_detach) <= 0 {
		err = fmt.Errorf("expected InfoSpaceDetach result to be non-empty, but is %v\n", r_detach)
		return
	}

	r_isDetached, err = is.InfoSpaceIsDetached()
	if err != nil {
		return
	}
	if !r_isDetached {
		err = fmt.Errorf("expected InfoSpaceIsDetached to be true, but is %v\n", r_isDetached)
		return
	}

	/*
	 * re-attach to detached index (should recover index)
	 */
	fmt.Printf("testInfoSpaceLifecycle: re-attach to detached index (should recover index)\n")

	_, err = is.InfoSpaceAttach()
	if err != nil {
		return
	}

	r_isDetached, err = is.InfoSpaceIsDetached()
	if err != nil {
		return
	}
	if r_isDetached {
		err = fmt.Errorf("expected InfoSpaceIsDetached to be false, but is %v\n", r_isDetached)
		return
	}

	r_window, err = is.InfoSpaceGetWindow()
	if err != nil {
		return
	}
	if r_window <= 0 {
		err = fmt.Errorf("expected InfoSpaceGetWindow to be non-zero, but is %v\n", r_window)
		return
	}

	r_windowEnd, err = is.InfoSpaceGetWindowEnd()
	if err != nil {
		return
	}
	if r_windowEnd <= 0 {
		err = fmt.Errorf("expected InfoSpaceGetWindowEnd to be non-zero, but is %v\n", r_windowEnd)
		return
	}

	///////////////////////////////////////////////////////////////////////////
	// now test adding document to the index
	///////////////////////////////////////////////////////////////////////////

	//
	// indexing documents provide a consistent interface using addDoc that
	// expects a document via filename or string, a boolean true => document
	// is provided ed as string.
	//
	// adding both file and string documents are handled through addDoc,
	// which returns -2 if document is duplicate, -1 if success.
	// this is because include/vank/IndexEnvironment.hpp declares addFile()
	// as void and addString as returning docID, and we want consistent
	// interface that also indicates whether the document is a duplicate.
	//

	//fmt.Printf("testInfoSpaceLifecycle: add document as file, should return a docId of -1\n")
	fp, err := filepath.Abs("data/txdoc1.html")
	if err != nil {
		return
	}

	r_docId, err := is.InfoSpaceAddDoc(fp, false)
	if DocIDToInt(r_docId) != -1 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: with InfoSpaceAddFile r_docId %v err %v\n", DocIDToInt(r_docId), err)
		return
	}

	// add document as string
	var docno1 string = `    <docno>offer-uidf5a8b61dacd606099e941250f695b21buid-f32d3adba31e486dc1ead5978ef6d7d39edb1393a2fe04d679b0adbb790d80140c9b7fbbe2081342c8157613dc56a80527bfb029bc42ac3b3d6e3dc7250ddb1f</docno>
`
	var docno2 string = `    <docno>offer-uidf5a8b61dacd606099e941250f695b22buid-f32d3adba31e486dc1ead5978ef6d7d39edb1393a2fe04d679b0adbb790d80140c9b7fbbe2081342c8157613dc56a80527bfb029bc42ac3b3d6e3dc7250ddb1f</docno>
`
	var common_preamble = `
<document>
  <meta>
    <odmver>1</odmver>
    <schver>1</schver>
    <base-time>2016-09-24 14:42:51 UTC</base-time>
    <curr-epoch>B2011</curr-epoch>
    <prev-epoch>A2010</prev-epoch>
    <max-areas>63</max-areas>
    <max-cats>63</max-cats>
    <app>vectavi</app>
`
	var common_rest = `    <docver>f32d3adba31e486dc1ead5978ef6d7d39edb1393a2fe04d679b0adbb790d80140c9b7fbbe2081342c8157613dc56a80527bfb029bc42ac3b3d6e3dc7250ddb1f</docver>
  </meta>
  <tx>
    <tx-tim>
      <tx-tim-c1>0</tx-tim-c1>
      <tx-tim-c2>1050</tx-tim-c2>
      <tx-tim-c3>18</tx-tim-c3>
    </tx-tim>
    <tx-uid>uidf5a8b61dacd606099e941250f695b21buid</tx-uid>
    <tx-ipa>50.241.107.77</tx-ipa>
    <tx-cls>offer</tx-cls>
    <tx-sts>active</tx-sts>
    <tx-prv>secret</tx-prv>
    <tx-obj>badfood</tx-obj>
    <tx-sty>declared</tx-sty>
    <tx-src>anonymous</tx-src>
    <tx-brn>badfood</tx-brn>
    <tx-ttl>badfood</tx-ttl>
    <tx-adr>badfood</tx-adr>
    <tx-lat>
      <tx-lat-c1>0</tx-lat-c1>
      <tx-lat-c2>0</tx-lat-c2>
      <tx-lat-c3>0</tx-lat-c3>
      <tx-lat-c4>0</tx-lat-c4>
    </tx-lat>
    <tx-lon>
      <tx-lon-c1>0</tx-lon-c1>
      <tx-lon-c2>0</tx-lon-c2>
      <tx-lon-c3>0</tx-lon-c3>
      <tx-lon-c4>0</tx-lon-c4>
    </tx-lon>
    <tx-ops>
      <tx-ops-nam-cls>topic</tx-ops-nam-cls>
      <tx-ops-nam-top-des>verifying media upload with beta deployment </tx-ops-nam-top-des>
      <tx-ops-nam-top-cat>issue</tx-ops-nam-top-cat>
      <tx-ops-nam-top-med>
        <tx-ops-nam-top-med-cls>photo</tx-ops-nam-top-med-cls>
        <tx-ops-nam-top-med-len>1</tx-ops-nam-top-med-len>
        <tx-ops-nam-top-med-pth>/home/tavit/Work/repo/media/z9QYGirj3nb3v_-qdE0_8g.png</tx-ops-nam-top-med-pth>
        <tx-ops-nam-top-med-cls>video</tx-ops-nam-top-med-cls>
        <tx-ops-nam-top-med-tim>
          <tx-ops-nam-top-med-tim-c1>0</tx-ops-nam-top-med-tim-c1>
          <tx-ops-nam-top-med-tim-c2>0</tx-ops-nam-top-med-tim-c2>
        </tx-ops-nam-top-med-tim>
        <tx-ops-nam-top-med-pth>/home/tavit/Work/repo/media/zigmGfyL3l81qQZ77MkC_Q.MOV</tx-ops-nam-top-med-pth>
      </tx-ops-nam-top-med>
      <tx-ops-nam-top-sbs>
        <tx-ops-nam-top-sbs-cls>badfood</tx-ops-nam-top-sbs-cls>
        <tx-ops-nam-top-sbs-nam>test</tx-ops-nam-top-sbs-nam>
      </tx-ops-nam-top-sbs>
      <tx-ops-nam-cls>time</tx-ops-nam-cls>
      <tx-ops-nam-tim-lif>
        <tx-ops-nam-tim-lif-c1>0</tx-ops-nam-tim-lif-c1>
        <tx-ops-nam-tim-lif-c2>15</tx-ops-nam-tim-lif-c2>
        <tx-ops-nam-tim-lif-c3>0</tx-ops-nam-tim-lif-c3>
      </tx-ops-nam-tim-lif>
      <tx-ops-nam-cls>price</tx-ops-nam-cls>
      <tx-ops-nam-prc-xch>offer</tx-ops-nam-prc-xch>
      <tx-ops-nam-prc-por>
        <tx-ops-nam-prc-por-c1>0</tx-ops-nam-prc-por-c1>
        <tx-ops-nam-prc-por-c2>0</tx-ops-nam-prc-por-c2>
        <tx-ops-nam-prc-por-c3>0</tx-ops-nam-prc-por-c3>
        <tx-ops-nam-prc-por-c4>0</tx-ops-nam-prc-por-c4>
      </tx-ops-nam-prc-por>
      <tx-ops-nam-cls>place</tx-ops-nam-cls>
      <tx-ops-nam-plc-via>member</tx-ops-nam-plc-via>
      <tx-ops-nam-plc-adr>cartwright lane  billerica  middlesex county  massachusetts  01821  united states of america</tx-ops-nam-plc-adr>
      <tx-ops-nam-plc-lon>
        <tx-ops-nam-plc-lon-c1>0</tx-ops-nam-plc-lon-c1>
        <tx-ops-nam-plc-lon-c2>-71</tx-ops-nam-plc-lon-c2>
        <tx-ops-nam-plc-lon-c3>-2588</tx-ops-nam-plc-lon-c3>
        <tx-ops-nam-plc-lon-c4>-5000</tx-ops-nam-plc-lon-c4>
      </tx-ops-nam-plc-lon>
      <tx-ops-nam-plc-lat>
        <tx-ops-nam-plc-lat-c1>0</tx-ops-nam-plc-lat-c1>
        <tx-ops-nam-plc-lat-c2>42</tx-ops-nam-plc-lat-c2>
        <tx-ops-nam-plc-lat-c3>5454</tx-ops-nam-plc-lat-c3>
        <tx-ops-nam-plc-lat-c4>7000</tx-ops-nam-plc-lat-c4>
      </tx-ops-nam-plc-lat>
      <tx-ops-nam-plc-rad>
        <tx-ops-nam-plc-rad-c1>4</tx-ops-nam-plc-rad-c1>
        <tx-ops-nam-plc-rad-c2>0</tx-ops-nam-plc-rad-c2>
      </tx-ops-nam-plc-rad>
      <tx-ops-nam-cls>member</tx-ops-nam-cls>
      <tx-ops-nam-mbr-cnt>1</tx-ops-nam-mbr-cnt>
      <tx-ops-nam-mbr-brn>vectavi</tx-ops-nam-mbr-brn>
      <tx-ops-nam-mbr-ttl>support</tx-ops-nam-mbr-ttl>
      <tx-ops-nam-mbr-nms>
        <tx-ops-nam-mbr-nms-nam-cls>declared</tx-ops-nam-mbr-nms-nam-cls>
        <tx-ops-nam-mbr-nms-nam-als>badfood</tx-ops-nam-mbr-nms-nam-als>
      </tx-ops-nam-mbr-nms>
    </tx-ops>
  </tx>
</document>

`

	var documentAsString string = common_preamble + docno1 + common_rest
	var documentAsString2 string = common_preamble + docno2 + common_rest

	//fmt.Printf("testInfoSpaceLifecycle: add document as string, should be a duplicate and return docId of -1\n")
	r_docId, err = is.InfoSpaceAddDoc(documentAsString, true)
	if DocIDToInt(r_docId) != -1 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: with InfoSpaceAddString r_docId %v err %v\n", DocIDToInt(r_docId), err)
		return
	}

	fmt.Printf("testInfoSpaceLifecycle: add document as string, should return a docId of 2\n")
	r_docId, err = is.InfoSpaceAddDoc(documentAsString2, true)
	if DocIDToInt(r_docId) != 2 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: with InfoSpaceAddString r_docId %v err %v\n", DocIDToInt(r_docId), err)
		return
	}
	fmt.Printf("testInfoSpaceLifecycle: add document as string, returned a docId of %v\n", DocIDToInt(r_docId))

	// need a new method to do this as the docID is not returned
	// hack for now
	var docId = IntToDocID(2)

	// check that last added document is in the index
	r_domainExists, err := is.InfoSpaceDomainExists(docId)
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceDomainExists for id %v, error %v\n", DocIDToInt(r_docId), err)
		return
	}
	if !r_domainExists {
		err = fmt.Errorf("expected document id %v to exist\n", DocIDToInt(r_docId))
		return
	}

	// fetch a document from the index by its id.
	r_domain, err := is.InfoSpaceGetDomain(docId)
	if r_domain == nil || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: with r_domain %v err %v\n", r_domain, err)
		return
	}
	defer DeleteDomain(r_domain)

	docstring, err := r_domain.DomainRenderToString()
	if len(docstring) <= 0 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: with docstring %v err %v\n", len(docstring), err)
		return
	}

	//fmt.Printf("testInfoSpaceLifecycle: rendered docstring length %v\n", len(docstring))
	//fmt.Printf("testInfoSpaceLifecycle: rendered documentAsString2 length %v\n", len(documentAsString2))
	//docstring = stripSpaces(docstring)
	//documentAsString2 = stripSpaces(documentAsString2)
	//fmt.Printf("testInfoSpaceLifecycle: trimmed docstring length %v\n", len(docstring))
	//fmt.Printf("testInfoSpaceLifecycle: trimmed documentAsString2 length %v\n", len(documentAsString2))
	//fmt.Printf("testInfoSpaceLifecycle: trimmed docstring %s\n", docstring)
	//fmt.Printf("\n")
	//fmt.Printf("testInfoSpaceLifecycle: trimmed documentAsString2 %s\n", documentAsString2)

	// verify retrieved document content
	var i int
	var j, size int64
	var rometa, rwmeta, fields StringPairVector
	var pair Wrapped_StringPair
	var key, value string

	rometa, err = r_domain.DomainGetROMeta()
	if rometa == nil || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetROMeta with rometa %v err %v\n", rometa, err)
		return
	}

	i = 0
	j = 0
	size = rometa.Size()
	//fmt.Printf("\ntestInfoSpaceLifecycle: ROMeta size = %v\n", size)
	for j < size {
		pair = rometa.Get(i)
		if pair != nil {
			key = pair.GetFirst()
			value = pair.GetSecond()
			fmt.Printf("testInfoSpaceLifecycle: ROMeta key = %v, value = %v\n", key, value)
		}
		i += 1
		j += 1
	}
	DeleteStringPairVector(rometa)

	rwmeta, err = r_domain.DomainGetRWMeta()
	if rwmeta == nil || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetRWMeta with rwmeta %v err %v\n", rwmeta, err)
		return
	}

	i = 0
	j = 0
	size = rwmeta.Size()
	//fmt.Printf("\ntestInfoSpaceLifecycle: RWMeta size = %v\n", size)
	for j < size {
		pair = rwmeta.Get(i)
		if pair != nil {
			key = pair.GetFirst()
			value = pair.GetSecond()
			fmt.Printf("testInfoSpaceLifecycle: RWMeta key = %v, value = %v\n", key, value)
		}
		i += 1
		j += 1
	}
	DeleteStringPairVector(rwmeta)

	fields, err = r_domain.DomainGetFields()
	if fields == nil || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetFields with fields %v err %v\n", fields, err)
		return
	}

	i = 0
	j = 0
	size = fields.Size()
	//fmt.Printf("\ntestInfoSpaceLifecycle: fields size = %v\n", size)
	for j < size {
		pair = fields.Get(i)
		if pair != nil {
			key = pair.GetFirst()
			value = pair.GetSecond()
			//fmt.Printf("testInfoSpaceLifecycle: fields key = %v, value = %v\n", key, value)
		}
		i += 1
		j += 1
	}
	DeleteStringPairVector(fields)

	///////////////////////////////////////////////////////////////////////////
	// now test indexing a hand constructed document
	///////////////////////////////////////////////////////////////////////////
	r_domain2, err := is.NewDomain()
	if r_domain2 == nil || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed NewDomain with r_domain2 %v err %v\n", r_domain2, err)
		return
	}
	defer DeleteDomain(r_domain2)

	// ROMeta field name and value to set
	var romfn, romfv string = "app", "infospace_test.go"

	// RWMeta field name and value to set
	var rwmfn, rwmfv string = "docno", "test-document-no-uidf5a8"
	var rwmfn2, rwmfv2 string = "docver", "1"

	// fields field name and value to set
	var fd1n, fd1v string = "tx", ""
	// tx domain_base [common] fields
	var fd2n, fd2v string = "tx-cls", "offer"
	var fd3n, fd3v string = "tx-sts", "active"
	var fd4n, fd4v string = "tx-prv", "secret"
	var fd5n, fd5v string = "tx-obj", "badfood"
	var fd6n, fd6v string = "tx-sty", "declared"
	var fd7n, fd7v string = "tx-src", "anonymous"

	var was string

	// test metadata set path

	was, err = r_domain2.DomainSetMeta(romfn, "badfood")
	// ro metadata is not settable, should return existing value
	if was != romfv || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainSetMeta with expected %v was %v err %v\n", romfv, was, err)
		return
	}

	was, err = r_domain2.DomainSetMeta(rwmfn, rwmfv)
	// rw metadata is settable, should return set value
	if was != rwmfv || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainSetMeta with expected %v was %v err %v\n", rwmfv, was, err)
		return
	}

	was, err = r_domain2.DomainSetMeta(rwmfn2, rwmfv2)
	// rw metadata is settable, should return set value
	if was != rwmfv2 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainSetMeta with expected %v was %v err %v\n", rwmfv2, was, err)
		return
	}

	// test metadata get path

	was, err = r_domain2.DomainGetMeta(romfn)
	if was != romfv || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetMeta with was %v err %v\n", was, err)
		return
	}

	was, err = r_domain2.DomainGetMeta(rwmfn)
	if was != rwmfv || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetMeta with was %v err %v\n", was, err)
		return
	}

	was, err = r_domain2.DomainGetMeta(rwmfn2)
	if was != rwmfv2 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetMeta with was %v err %v\n", was, err)
		return
	}

	// test field add path

	/*
	   	TODO: DomainGetCursor / DomainSetCursor should not be public
	   	if a domain is persisted, then can use DomainSetField to modify field,
	   	otherwise use DomainAddField to add a field under a parent ordinal.
	   	see usage below.

	   	j = 0
	   	var nCur int64

	   	nCur, err = r_domain2.DomainSetCursor(j)
	   	if nCur != j || err != nil {
	   		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainSetCursor with nCur %v err %v\n", nCur, err)
	       	return
	   	}

	   	nCur, err = r_domain2.DomainGetCursor()
	   	if nCur != j || err != nil {
	   		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetCursor with nCur %v err %v\n", nCur, err)
	       	return
	   	}

	*/

	//
	// field count should be zero for a new document,
	// and represents initial parent ordinal
	//
	i, err = r_domain2.DomainGetFieldCount()
	if i > 0 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetFieldCount expected %v was %v err %v\n", 0, i, err)
		return
	}

	was, err = r_domain2.DomainAddField(fd1n, fd1v, i)
	if was != fd1v || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainAddField with expected %v was %v err %v\n", fd1v, was, err)
		return
	}

	i, err = r_domain2.DomainGetFieldCount()
	if i > 1 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetFieldCount expected %v was %v err %v\n", 1, i, err)
		return
	}

	was, err = r_domain2.DomainAddField(fd2n, fd2v, i)
	if was != fd2v || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainAddField with expected %v was %v err %v\n", fd2v, was, err)
		return
	}

	was, err = r_domain2.DomainAddField(fd3n, fd3v, i)
	if was != fd3v || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainAddField with expected %v was %v err %v\n", fd3v, was, err)
		return
	}

	was, err = r_domain2.DomainAddField(fd4n, fd4v, i)
	if was != fd4v || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainAddField with expected %v was %v err %v\n", fd4v, was, err)
		return
	}

	was, err = r_domain2.DomainAddField(fd5n, fd5v, i)
	if was != fd5v || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainAddField with expected %v was %v err %v\n", fd5v, was, err)
		return
	}

	was, err = r_domain2.DomainAddField(fd6n, fd6v, i)
	if was != fd6v || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainAddField with expected %v was %v err %v\n", fd6v, was, err)
		return
	}

	was, err = r_domain2.DomainAddField(fd7n, fd7v, i)
	if was != fd7v || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainAddField with expected %v was %v err %v\n", fd7v, was, err)
		return
	}

	i, err = r_domain2.DomainGetFieldCount()
	if i != 7 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetFieldCount expected %v was %v err %v\n", 0, i, err)
		return
	}

	// print the document fields for review
	fields, err = r_domain2.DomainGetFields()
	if fields == nil || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetFields with fields %v err %v\n", fields, err)
		return
	}

	i = 0
	j = 0
	size = fields.Size()
	//fmt.Printf("\ntestInfoSpaceLifecycle: fields size = %v\n", size)
	for j < size {
		pair = fields.Get(i)
		if pair != nil {
			key = pair.GetFirst()
			value = pair.GetSecond()
			//fmt.Printf("testInfoSpaceLifecycle: fields key = %v, value = %v\n", key, value)
		}
		i += 1
		j += 1
	}
	DeleteStringPairVector(fields)

	// test field set path - modify added field values

	fd3v = "inactive"
	was, err = r_domain2.DomainSetField(fd3n, fd3v, i)
	if was != fd3v || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainSetField with expected %v was %v err %v\n", fd3v, was, err)
		return
	}

	fd4v = "public"
	was, err = r_domain2.DomainSetField(fd4n, fd4v, i)
	if was != fd4v || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainSetField with expected %v was %v err %v\n", fd4v, was, err)
		return
	}

	// test field get path - verify modified field values

	was, err = r_domain2.DomainGetField(fd3n, 0)
	if was != fd3v || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetField with expected %v was %v err %v\n", fd3v, was, err)
		return
	}

	was, err = r_domain2.DomainGetField(fd4n, 0)
	if was != fd4v || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: failed DomainGetField with expected %v was %v err %v\n", fd4v, was, err)
		return
	}

	docstring, err = r_domain2.DomainRenderToString()
	if len(docstring) <= 0 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: with docstring %v err %v\n", len(docstring), err)
		return
	}

	//fmt.Printf("testInfoSpaceLifecycle: rendered docstring length %v\n", len(docstring))
	//docstring = stripSpaces(docstring)
	//fmt.Printf("testInfoSpaceLifecycle: trimmed docstring length %v\n", len(docstring))
	//fmt.Printf("testInfoSpaceLifecycle: trimmed docstring %s\n", docstring)
	//fmt.Printf("\n")

	// test save to index

	did, err := r_domain2.DomainSave()
	if err != nil {
		err = fmt.Errorf("failed DomainSave error %v\n", err)
		return
	}
	if DocIDToInt(did) <= 0 {
		err = fmt.Errorf("expected DomainSave id to be non-zero, id = %v\n", DocIDToInt(did))
		return
	}

	// test save to file

	var fn string = "doc2." + corpusFileClass
	fmt.Printf("testInfoSpaceLifecycle: fn =  %v\n", fn)
	nam, err := r_domain2.DomainSaveToFile(fn)
	if err != nil {
		err = fmt.Errorf("failed DomainSaveToFile error %v\n", err)
		return
	}
	f, err := os.Open(nam)
	if err != nil {
		err = fmt.Errorf("failed to open file created by DomainSaveToFile, savedPath = %v", nam)
		return
	}
	err = f.Close()
	if err != nil {
		return
	}

	///////////////////////////////////////////////////////////////////////////
	// now test document retrieval
	///////////////////////////////////////////////////////////////////////////

	// let's setup a DocIdVector to retrieve the 2 documents we added
	var docId1, docId2 int = 1, 2

	docIds, err := NewWrapped_DocIdVector(2)
	if err != nil {
		err = fmt.Errorf("failed to create DocIdVector err: %v", err)
		return
	}
	defer DeleteWrapped_DocIdVector(docIds)
	//fmt.Printf("testInfoSpaceLifecycle: DocIdVector size() %v\n", docIds.Size())

	err = docIds.WSet(0, IntToDocID(docId1))
	if err != nil {
		err = fmt.Errorf("failed to set doc id index 0 error %v\n", err)
		return
	}

	err = docIds.WSet(1, IntToDocID(docId2))
	if err != nil {
		err = fmt.Errorf("failed to set doc id index 1 error %v\n", err)
		return
	}

	// verify the DocIdVector is properly setup

	var v Lemur_api_DOCID_T
	i = 0
	j = 0
	for j < docIds.Size() {
		v, err = docIds.WGet(i)
		if err != nil {
			err = fmt.Errorf("failed to get doc id at index %v error %v\n", i, err)
			return
		}
		//fmt.Printf("testInfoSpaceLifecycle: docId at index %v = %v\n", i, DocIDToInt(v))

		// check that document is in the index
		r_domainExists, err = is.InfoSpaceDomainExists(v)
		if err != nil {
			err = fmt.Errorf("failed InfoSpaceDomainExists for id %v, error %v\n", DocIDToInt(v), err)
			return
		}
		if !r_domainExists {
			err = fmt.Errorf("expected document id %v to exist\n", DocIDToInt(v))
			return
		}

		i += 1
		j += 1
	}

	//
	// now retrieve the two documents in one call
	// Note: this returns a vector of poiners to Wrapped_Domain not Domain
	//
	// Note: when instantiating infospace objects in GO, we use the protected
	// flavor of interface functions. When infospace objects are instantiated
	// within c++, we use the unprotected (may throw exception) functions.
	//
	domains, err := is.InfoSpaceGetDomains(docIds)
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceGetDomains error %v\n", err)
		return
	}
	if domains.Size() != 2 {
		err = fmt.Errorf("failed InfoSpaceGetDomains expecting to retrieve %v documents\n", docIds.Size())
		return
	}

	i = 0
	j = 0
	//var r_d Domain
	var r_d Wrapped_Domain
	var dstring string
	for j < domains.Size() {
		// fetch document at index 0.
		r_d = domains.Get(i)
		if r_d == nil || err != nil {
			err = fmt.Errorf("testInfoSpaceLifecycle: with r_d %v err %v\n", r_d, err)
			return
		}

		//dstring, err = r_d.DomainRenderToString()
		dstring = r_d.RenderToString()
		if len(dstring) <= 0 || err != nil {
			err = fmt.Errorf("testInfoSpaceLifecycle: with dstring %v err %v\n", len(dstring), err)
			return
		}
		//fmt.Printf("testInfoSpaceLifecycle: rendered dstring length %v\n", len(dstring))
		//fmt.Printf("testInfoSpaceLifecycle: rendered dstring %s\n", dstring)

		// don't leak Domain at index
		DeleteWrapped_Domain(r_d)

		i += 1
		j += 1
	}
	// Domains have been freed, reclaim vector
	domains.Clear()
	DeleteDomainPtrVector(domains)

	///////////////////////////////////////////////////////////////////////////
	//
	// now test find extents
	//
	// reference:
	// 		vectavi/app/models/query.rb
	//		vectavi/app/models/domain_base.rb
	//		vectavi/app/models/domain_tx.rb
	//
	///////////////////////////////////////////////////////////////////////////
	var p string = "tx" // container tag

	// field for use in findFieldExtent
	var f1 string = "-ops-nam-top-des"     // subject_type container field
	var f2 string = "-ops-nam-top-cat"     // category container field
	var f3 string = "-ops-nam-top-sbs-nam" // subject_name container field

	// punctuaion and control characters to filter when sanitizing token
	var punct string = "!-/:-@[-`{-~"
	var contr string = "\x00-\x1F\x7F"

	// wildcard query prefix token
	var t1 string = punct + contr + "ver" + punct + contr + "if" + punct + contr
	var t2 string = punct + contr + "iss" + punct + contr
	var t3 string = punct + contr + "tes" + punct + contr

	// expected matching extent
	var v1 string = "verifying media upload with beta deployment"
	var v2 string = "issue"
	var v3 string = "test"

	var q1, q2, q3 string // built up query string
	var max int64 = 10    // max number of query results

	q1 = field_query(p, f1, t1)
	q2 = field_query(p, f2, t2)
	//q = "#band( #any(tx-cls) #od1( #wildcard(verif) ).tx-ops-nam-top-des )"
	//q = "#band( #any(tx-cls) #od1( #wildcard(iss) ).tx-ops-nam-top-cat )"
	//q = "#od1( #wildcard(iss) ).tx-ops-nam-top-cat"
	//q = "#wildcard(iss).tx-ops-nam-top-cat"
	//f2 = "-cls"
	//q = "#wildcard(off).tx-cls"
	q3 = field_query(p, f3, t3)

	var x1, x2, x3 StringVector

	x1, err = is.InfoSpaceFindExtent(q1, (p + f1), max)
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceFindExtent query %v error %v\n", q1, err)
		return
	}
	x2, err = is.InfoSpaceFindExtent(q2, (p + f2), max)
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceFindExtent query %v error %v\n", q2, err)
		return
	}
	x3, err = is.InfoSpaceFindExtent(q3, (p + f3), max)
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceFindExtent query %v error %v\n", q3, err)
		return
	}

	if x1 != nil {
		defer DeleteStringVector(x1)

		if x1.IsEmpty() || x1.Size() < 1 {
			err = fmt.Errorf("failed to match extent 1 = %v\n", v1)
			return
		}

		// remove duplicates from list
		// filter badfood from list
		// filter extents not starting with token (for autocompleter applications)

		if !x1.IsEmpty() {
			i = 0
			j = 0
			for j < x1.Size() {
				if x1.Get(i) != v1 {
					err = fmt.Errorf("failed to match extent 1 index %v return = %v\n", i, v1)
					return
				}
				i += 1
				j += 1
			}
		}
	}

	if x2 != nil {
		defer DeleteStringVector(x2)

		if x2.IsEmpty() || x2.Size() < 1 {
			err = fmt.Errorf("failed to match extent 2 = %v\n", v2)
			return
		}

		// remove duplicates from list
		// filter badfood from list
		// filter extents not starting with token (for autocompleter applications)

		if !x2.IsEmpty() {
			i = 0
			j = 0
			for j < x2.Size() {
				if x2.Get(i) != v2 {
					err = fmt.Errorf("failed to match extent 2 index %v return = %v\n", i, v2)
					return
				}
				i += 1
				j += 1
			}
		}
	}

	if x3 != nil {
		defer DeleteStringVector(x3)

		if x3.IsEmpty() || x3.Size() < 1 {
			err = fmt.Errorf("failed to match extent 3 = %v\n", v3)
			return
		}

		// remove duplicates from list
		// filter badfood from list
		// filter extents not starting with token (for autocompleter applications)

		if !x3.IsEmpty() {
			i = 0
			j = 0
			for j < x3.Size() {
				if x3.Get(i) != v3 {
					err = fmt.Errorf("failed to match extent 3 index %v return = %v\n", i, v3)
					return
				}
				i += 1
				j += 1
			}
		}
	}

	///////////////////////////////////////////////////////////////////////////
	// now test find, along with query numeric field and cursor functions
	///////////////////////////////////////////////////////////////////////////
	var q, low, high string

	// canonical radius is in meters
	low = "30000"
	high = "50000"

	q = radius_range(is, low, high)
	docIdVec, err := is.InfoSpaceFind(q)
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceFind query %v error %v\n", q, err)
		return
	}
	defer DeleteDocIdVector(docIdVec)

	// we expect the query to find 2 document with ids 2 and 1 at index 0 and 1
	if docIdVec.Size() != 2 {
		err = fmt.Errorf("expected 2 but found %v documents by radius_range\n", docIdVec.Size())
		return
	}

	// list ids of found documents
	//var v Lemur_api_DOCID_T
	i = 0
	j = 0
	for j < docIdVec.Size() {
		v = docIdVec.Get(i)

		if i == 0 && DocIDToInt(v) != 2 {
			err = fmt.Errorf("expected docid 2 but found %v at index %v for radius_range\n", DocIDToInt(v), i)
			return
		}
		if i == 1 && DocIDToInt(v) != 1 {
			err = fmt.Errorf("expected docid 1 but found %v at index %v for radius_range\n", DocIDToInt(v), i)
			return
		}

		i += 1
		j += 1
	}

	q = radius_between(is, low, high)
	docIdVec2, err := is.InfoSpaceFind(q)
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceFind query %v error %v\n", q, err)
		return
	}
	defer DeleteDocIdVector(docIdVec2)

	// we expect the query to find 2 document with ids 2 and 1 at index 0 and 1
	if docIdVec2.Size() != 2 {
		err = fmt.Errorf("expected 2 but found %v documents by radius_between\n", docIdVec2.Size())
		return
	}

	// list ids of found documents
	//var v Lemur_api_DOCID_T
	i = 0
	j = 0
	for j < docIdVec2.Size() {
		v = docIdVec2.Get(i)

		if i == 0 && DocIDToInt(v) != 2 {
			err = fmt.Errorf("expected docid 2 but found %v at index %v for radius_between\n", DocIDToInt(v), i)
			return
		}
		if i == 1 && DocIDToInt(v) != 1 {
			err = fmt.Errorf("expected docid 1 but found %v at index %v for radius_between\n", DocIDToInt(v), i)
			return
		}

		i += 1
		j += 1
	}

	j = 1
	j, err = is.InfoSpaceSetMax(j)
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceSetMax j %v error %v\n", j, err)
		return
	}
	k, err := is.InfoSpaceGetMax()
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceGetMax error %v\n", err)
		return
	}
	if k != j {
		err = fmt.Errorf("unexpected InfoSpaceGetMax value %v\n", k)
		return
	}

	q = radius_between(is, low, high)
	docIdVec3, err := is.InfoSpaceFind(q)
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceFind query %v error %v\n", q, err)
		return
	}
	defer DeleteDocIdVector(docIdVec3)

	if docIdVec3.Size() < j {
		err = fmt.Errorf("expected to find a document\n")
		return
	}
	v = docIdVec3.Get(0)
	if DocIDToInt(v) != 2 {
		err = fmt.Errorf("expected docid 2 but found %v in docIdVec3\n", DocIDToInt(v))
		return
	}

	k, err = is.InfoSpaceGetPos()
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceGetPos error %v\n", err)
		return
	}
	if k != 0 {
		err = fmt.Errorf("unexpected InfoSpaceGetPos value %v\n", k)
		return
	}

	j = 1
	j, err = is.InfoSpaceSetPos(j)
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceSetPos j %v error %v\n", j, err)
		return
	}

	docIdVec4, err := is.InfoSpaceFind(q)
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceFind query %v error %v\n", q, err)
		return
	}
	defer DeleteDocIdVector(docIdVec4)

	if docIdVec4.Size() < j {
		err = fmt.Errorf("expected to find a document\n")
		return
	}
	v = docIdVec4.Get(0)
	// TODO: when backend infospace _cursor is wired, we expect 1
	//if DocIDToInt(v) != 1 {
	if DocIDToInt(v) != 2 {
		err = fmt.Errorf("expected docid 1 but found %v in docIdVec4\n", DocIDToInt(v))
		return
	}

	k, err = is.InfoSpaceGetPos()
	if err != nil {
		err = fmt.Errorf("failed InfoSpaceGetPos error %v\n", err)
		return
	}
	if k != 1 {
		err = fmt.Errorf("unexpected InfoSpaceGetPos value %v\n", k)
		return
	}

	///////////////////////////////////////////////////////////////////////////
	// now delete the indexed documents
	///////////////////////////////////////////////////////////////////////////
	i = 0
	j = 0
	for j < docIds.Size() {
		v, err = docIds.WGet(i)
		if err != nil {
			err = fmt.Errorf("failed to get doc id at index %v error %v\n", i, err)
			return
		}

		// check that document is in the index
		r_domainExists, err = is.InfoSpaceDomainExists(v)
		if err != nil {
			err = fmt.Errorf("failed InfoSpaceDomainExists for id %v, error %v\n", DocIDToInt(v), err)
			return
		}
		if !r_domainExists {
			err = fmt.Errorf("expected document id %v to exist\n", DocIDToInt(v))
			return
		}

		v, err = is.InfoSpaceDeleteDomain(v)
		if err != nil {
			err = fmt.Errorf("failed to delete doc at docIds vector position %v error %v\n", i, err)
			return
		}

		i += 1
		j += 1
	}

	///////////////////////////////////////////////////////////////////////////
	// index should have one document now, and one file saved at corpus path
	///////////////////////////////////////////////////////////////////////////

	fmt.Printf("testInfoSpaceLifecycle: completed with infospace %v err %v\n", is, err)

	return

}

func sanitized(s string) (ss string) {
	//
	// from ruby query.rb
	//    def self.sanitized( s )
	//      re=/([[:punct:]]|[[:cntrl:]])/
	//      s.gsub(re, ' ') if s
	//    end
	//
	re := regexp.MustCompile(`([[:punct:]]|[[:cntrl:]])`)
	ss = re.ReplaceAllString(s, "")

	//fmt.Printf("sanitized: s = %v, ss = %v\n", s, ss)

	return
}

func field_query(p, f, token string) (q string) {
	q = " #band("
	q += " #any(" + p + "-cls)"
	q += " #od1( " + sanitized(token) + "* )." + p + f
	q += " )"
	return
}

// radius in range [low,high]
func radius_range(is InfoSpace, low, high string) (q string) {
	q = num_field_range(is, "tx-ops-nam-plc-rad", low, high)
	return
}

// radius between [low,high]
func radius_between(is InfoSpace, low, high string) (q string) {
	q = num_field_between(is, "tx-ops-nam-plc-rad", low, high)
	return
}

// match named numeric field with value in range [low,high]
func num_field_range(is InfoSpace, fn, lo, hi string) (q string) {

	// query vocabulary
	//var a, b, c, d, e, l, s string
	var b, c, s string

	//a=" #any( "
	b = " #band( "
	c = " )"
	//d=" #1( "
	//e=" #equals( "
	//l=" #less( "
	s = " #syn( "

	// query clause
	var c1, c2 string

	c1 = s + s + is.NFGreater(fn, lo) + c + s + is.NFEquals(fn, lo) + c + c
	c2 = s + s + is.NFLess(fn, hi) + c + s + is.NFEquals(fn, hi) + c + c

	q = b + c1 + c2 + c

	return
}

// match named numeric field with value between [low,high]
func num_field_between(is InfoSpace, fn, lo, hi string) (q string) {

	// query vocabulary
	//var a, b, c, d, e, l, s string
	var c, s string

	//a=" #any( "
	//b=" #band( "
	c = " )"
	//d=" #1( "
	//e=" #equals( "
	//l=" #less( "
	s = " #syn( "

	q = s + is.NFBetween(fn, lo, hi) + c

	return
}
